unit UFormMain;

{$mode Delphi}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, SynEdit, Forms, Controls, Graphics, Dialogs,
  Menus, ActnList, StdCtrls, ComCtrls, ExtCtrls,
  StrUtils, SpecializedList, UTarget,
  Registry, URegistry, Process;

type

  { TMainForm }

  TMainForm = class(TForm)
    AProgramShowExecutionPoint: TAction;
    AShowSourcePosition: TAction;
    AShowTargetPosition: TAction;
    AProgramCompile: TAction;
    AAbout: TAction;
    ABreakpointToggle: TAction;
    AViewCompilers: TAction;
    AOptions: TAction;
    AProgramStepOut: TAction;
    AProgramRunToCursor: TAction;
    AProgramStepOver: TAction;
    AProgramStepInto: TAction;
    AProgramPause: TAction;
    AProgramStop: TAction;
    AHelp: TAction;
    AProgramRun: TAction;
    AProjectNew: TAction;
    AExit: TAction;
    AProjectSaveAs: TAction;
    AProjectClose: TAction;
    AProjectSave: TAction;
    AProjectOpen: TAction;
    ActionList1: TActionList;
    ImageListMain: TImageList;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem10: TMenuItem;
    MenuItem11: TMenuItem;
    MenuItem12: TMenuItem;
    MenuItem13: TMenuItem;
    MenuItem14: TMenuItem;
    MenuItem15: TMenuItem;
    MenuItem16: TMenuItem;
    MenuItem17: TMenuItem;
    MenuItem18: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem22: TMenuItem;
    MenuItem24: TMenuItem;
    MenuItem25: TMenuItem;
    MenuItem27: TMenuItem;
    MenuItem28: TMenuItem;
    MenuItem30: TMenuItem;
    MenuItem31: TMenuItem;
    MenuItem32: TMenuItem;
    MenuItem33: TMenuItem;
    MenuItem34: TMenuItem;
    MenuItem35: TMenuItem;
    MenuItemOpenRecent: TMenuItem;
    MenuItemTarget: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItem8: TMenuItem;
    MenuItem9: TMenuItem;
    OpenDialog1: TOpenDialog;
    PageControlMain: TPageControl;
    PageControlRight: TPageControl;
    PageControlBottom: TPageControl;
    PanelLeft: TPanel;
    PanelMemory: TPanel;
    PanelCPU: TPanel;
    PanelOutput: TPanel;
    PanelInput: TPanel;
    PopupMenuTargets: TPopupMenu;
    PopupMenuRecentFiles: TPopupMenu;
    SaveDialog1: TSaveDialog;
    Splitter1: TSplitter;
    Splitter2: TSplitter;
    Splitter3: TSplitter;
    Splitter4: TSplitter;
    Splitter5: TSplitter;
    StatusBarMain: TStatusBar;
    TabSheetMessages: TTabSheet;
    TabSheetDebug: TTabSheet;
    TabSheetSource: TTabSheet;
    TabSheetTarget: TTabSheet;
    TimerRefresh: TTimer;
    ToolBarMain: TToolBar;
    ToolButton1: TToolButton;
    ToolButton10: TToolButton;
    ToolButton11: TToolButton;
    ToolButton12: TToolButton;
    ToolButton13: TToolButton;
    ToolButton14: TToolButton;
    ToolButton15: TToolButton;
    ToolButtonTarget: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    procedure ABreakpointToggleExecute(Sender: TObject);
    procedure AExitExecute(Sender: TObject);
    procedure AOptionsExecute(Sender: TObject);
    procedure AProgramCompileExecute(Sender: TObject);
    procedure AProgramPauseExecute(Sender: TObject);
    procedure AProgramRunToCursorExecute(Sender: TObject);
    procedure AProgramRunExecute(Sender: TObject);
    procedure AProgramShowExecutionPointExecute(Sender: TObject);
    procedure AProgramStepIntoExecute(Sender: TObject);
    procedure AProgramStepOutExecute(Sender: TObject);
    procedure AProgramStepOverExecute(Sender: TObject);
    procedure AProgramStopExecute(Sender: TObject);
    procedure AProjectCloseExecute(Sender: TObject);
    procedure AProjectNewExecute(Sender: TObject);
    procedure AProjectOpenExecute(Sender: TObject);
    procedure AProjectSaveAsExecute(Sender: TObject);
    procedure AProjectSaveExecute(Sender: TObject);
    procedure AShowSourcePositionExecute(Sender: TObject);
    procedure AShowTargetPositionExecute(Sender: TObject);
    procedure AViewCompilersExecute(Sender: TObject);
    procedure AViewInterpretterExecute(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TimerRefreshTimer(Sender: TObject);
  private
    FCurrentTarget: TTarget;
    procedure SetCurrentTarget(AValue: TTarget);
    procedure AProjectOpenRecentExecute(Sender: TObject);
    procedure TargetStateChanged(Sender: TObject);
    procedure MessagesChanged(Sender: TObject);
    procedure MenuItemTargetClick(Sender: TObject);
    procedure ProjectOpen(FileName: string);
    procedure UpdateToolBarHints;
    procedure DockInit;
    procedure LastOpenedListChange(Sender: TObject);
  public
    LastFileName: string;
    procedure LoadFromRegistry(Root: HKEY; Key: string);
    procedure SaveToRegistry(Root: HKEY; Key: string);
    procedure UpdateInterface;
    procedure UpdateStatusBar;
    procedure UpdateTargetList;
    property CurrentTarget: TTarget read FCurrentTarget write SetCurrentTarget;
  end;

var
  MainForm: TMainForm;


implementation

{$R *.lfm}

uses
  UCore, UFormCPU, UFormOptions, UFormMessages, UFormTargets, UFormOutput, UFormInput,
  UFormMemory, UFormSourceCode, UFormTargetCode, UTargetInterpretter;

resourcestring
  SProgramExited = 'Program exited';
  SProgramStarted = 'Program started';
  SProgramStopped = 'Program stopped';
  SNewProject = 'New project';
  SCompileStart = 'Compiling...';
  SDone = 'Done';

{ TMainForm }

procedure TMainForm.SetCurrentTarget(AValue: TTarget);
var
  I: Integer;
begin
  if FCurrentTarget = AValue then Exit;
  FCurrentTarget := AValue;
  for I := 0 to Core.Targets.Count - 1 do
    TTarget(Core.Targets[I]).OnChangeState := nil;
  FCurrentTarget.OnChangeState := TargetStateChanged;
  FCurrentTarget.Messages.OnChange := MessagesChanged;
  UpdateTargetList;
  UpdateInterface;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  with Core.ScaleDPI do begin
    ApplyToAll(DesignDPI);
    ScaleImageList(ImageListMain, DesignDPI);
  end;
  Core.CoolTranslator1.Translate;
  LoadFromRegistry(HKEY(Core.ApplicationInfo.RegistryRoot), Core.ApplicationInfo.RegistryKey);
  PageControlMain.TabIndex := 0;
  if Core.OpenProjectOnStart and (Core.LastOpenedList.Items.Count > 0) then
    ProjectOpen(Core.LastOpenedList.Items[0])
    else AProjectNew.Execute;
  UpdateInterface;
  UpdateToolBarHints;
  UpdateTargetList;
  DockInit;
end;

procedure TMainForm.TimerRefreshTimer(Sender: TObject);
begin
  FormCPU.Reload;
  FormOutput.Reload;
  FormMemory.Reload;
end;

procedure TMainForm.TargetStateChanged(Sender: TObject);
begin
  UpdateInterface;
  if CurrentTarget.State = rsPaused then
    AProgramShowExecutionPoint.Execute;
  if CurrentTarget.State = rsStopped then CurrentTarget.Messages.AddMessage(SProgramStopped);
end;

procedure TMainForm.MessagesChanged(Sender: TObject);
begin
  FormMessages.Reload;
end;

procedure TMainForm.MenuItemTargetClick(Sender: TObject);
var
  I: Integer;
begin
  if Sender is TMenuItem then
  with TMenuItem(Sender) do begin
    CurrentTarget := TTarget(Core.Targets[MenuIndex]);
    Checked := True;
    for I := 0 to Parent.Count - 1 do
      if Parent.Items[I].MenuIndex <> MenuIndex then Parent.Items[I].Checked := False;
  end;
end;

procedure TMainForm.ProjectOpen(FileName: string);
begin
  AProjectClose.Execute;
  FormSourceCode.MemoSource.Lines.LoadFromFile(UTF8Decode(FileName));
  Core.LastOpenedList.AddItem(FileName);
  Core.ProjectFileName := FileName;
  Core.Modified := False;
  CurrentTarget.Compiled := False;
  UpdateInterface;
end;

procedure TMainForm.UpdateToolBarHints;
var
  I: Integer;
begin
  for I := 0 to ToolBarMain.ControlCount - 1 do
    ToolBarMain.Controls[I].Hint := ToolBarMain.Controls[I].Caption;
end;

procedure TMainForm.DockInit;
begin
  FormSourceCode.ManualDock(TabSheetSource, nil, alClient);
  FormSourceCode.Align := alClient;
  FormSourceCode.Show;
  FormTargetCode.ManualDock(TabSheetTarget, nil, alClient);
  FormTargetCode.Align := alClient;
  FormTargetCode.Show;
  FormCPU.ManualDock(PanelCPU, nil, alClient);
  FormCPU.Align := alClient;
  FormCPU.Show;
  FormInput.ManualDock(PanelInput, nil, alClient);
  FormInput.Align := alClient;
  FormInput.Show;
  FormOutput.ManualDock(PanelOutput, nil, alClient);
  FormOutput.Align := alClient;
  FormOutput.Show;
  FormMemory.ManualDock(PanelMemory, nil, alClient);
  FormMemory.Align := alClient;
  FormMemory.Show;
  FormMessages.ManualDock(TabSheetMessages, nil, alClient);
  FormMessages.Align := alClient;
  FormMessages.Show;
  PageControlRight.Width := MainForm.Width div 2;
  PageControlBottom.Height := MainForm.Height div 5;
end;

procedure TMainForm.LastOpenedListChange(Sender: TObject);
begin
  Core.LastOpenedList.LoadToMenuItem(MenuItemOpenRecent, AProjectOpenRecentExecute);
  Core.LastOpenedList.LoadToMenuItem(PopupMenuRecentFiles.Items, AProjectOpenRecentExecute);
end;

procedure TMainForm.LoadFromRegistry(Root: HKEY; Key: string);
var
  TargetName: string;
begin
  with TRegistryEx.Create do
  try
    RootKey := Root;
    OpenKey(Key, True);
    TargetName := ReadStringWithDefault('TargetName', 'Interpretter');
    LastFileName := ReadStringWithDefault('LastFileName',
      ExtractFileDir(Application.ExeName) + DirectorySeparator + 'Examples');
    CurrentTarget := Core.Targets.FindByName(TargetName);
    if not Assigned(CurrentTarget) then CurrentTarget := TTarget(Core.Targets[0]);
  finally
    Free;
  end;
end;

procedure TMainForm.SaveToRegistry(Root: HKEY; Key: string);
begin
  with TRegistryEx.Create do
  try
    RootKey := Root;
    OpenKey(Key, True);
    WriteString('TargetName', CurrentTarget.Name);
    WriteString('LastFileName', LastFileName);
  finally
    Free;
  end;
end;

procedure TMainForm.UpdateInterface;
var
  Title: string;
begin
  AProjectSave.Enabled := Core.Modified;
  Title := Core.ApplicationInfo.AppName;
  if Core.ProjectFileName <> '' then Title := Core.ProjectFileName + ' - ' + Title;
  if Core.Modified then Title := Title + ' *';
  Caption := Title;
  FormSourceCode.MemoSource.Enabled := Core.ProjectFileName <> '';
  AProjectClose.Enabled := Core.ProjectFileName <> '';
  AProgramRun.Enabled := (tcRun in CurrentTarget.Capabilities) and
    (Core.ProjectFileName <> '') and (CurrentTarget.State = rsStopped) or (CurrentTarget.State = rsPaused);
  AProgramPause.Enabled := (tcPause in CurrentTarget.Capabilities) and
    (Core.ProjectFileName <> '') and (CurrentTarget.State = rsRunning);
  AProgramStop.Enabled := (tcStop in CurrentTarget.Capabilities) and
    (Core.ProjectFileName <> '') and (CurrentTarget.State <> rsStopped);
  AProgramCompile.Enabled := (tcCompile in CurrentTarget.Capabilities) and
    (Core.ProjectFileName <> '');
  AProgramStepInto.Enabled := (tcStepInto in CurrentTarget.Capabilities) and
    (Core.ProjectFileName <> '') and ((CurrentTarget.State = rsPaused) or (CurrentTarget.State = rsStopped));
  AProgramStepOut.Enabled := (tcStepOut in CurrentTarget.Capabilities) and
    (Core.ProjectFileName <> '') and (CurrentTarget.State = rsPaused);
  AProgramRunToCursor.Enabled := (tcRunToCursor in CurrentTarget.Capabilities) and
    (Core.ProjectFileName <> '') and ((CurrentTarget.State = rsPaused) or (CurrentTarget.State = rsStopped));
  AProgramStepOver.Enabled := (tcStepOver in CurrentTarget.Capabilities) and
    (Core.ProjectFileName <> '') and ((CurrentTarget.State = rsPaused) or (CurrentTarget.State = rsStopped));
  AProgramShowExecutionPoint.Enabled := (tcPause in CurrentTarget.Capabilities) and
    (Core.ProjectFileName <> '') and (CurrentTarget.State = rsPaused);
  AShowSourcePosition.Enabled := CurrentTarget.Compiled;
  AShowTargetPosition.Enabled := CurrentTarget.Compiled;
  UpdateStatusBar;
end;

procedure TMainForm.UpdateStatusBar;
begin
  StatusBarMain.Panels[0].Text := IntToStr(FormSourceCode.MemoSource.CaretPos.X) +
    ', ' + IntToStr(FormSourceCode.MemoSource.CaretPos.Y);
end;

procedure TMainForm.UpdateTargetList;
var
  I: Integer;
begin
  Core.Targets.LoadToMenuItem(MenuItemTarget, MenuItemTargetClick, CurrentTarget);
  Core.Targets.LoadToMenuItem(PopupMenuTargets.Items, MenuItemTargetClick, CurrentTarget);

  // Update toolbutton
  I := Core.Targets.IndexOf(CurrentTarget);
  if I <> -1 then
  with TTarget(Core.Targets[I]) do begin
    ToolButtonTarget.ImageIndex := ImageIndex;
    ToolButtonTarget.Caption := Name;
    ToolButtonTarget.OnClick := MenuItemTargetClick();
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  UpdateTargetList;
  Core.LastOpenedList.OnChange := LastOpenedListChange;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  SaveToRegistry(HKEY(Core.ApplicationInfo.RegistryRoot), Core.ApplicationInfo.RegistryKey);
end;

procedure TMainForm.AProgramRunExecute(Sender: TObject);
begin
  if CurrentTarget is TTargetInterpretter then
    TTargetInterpretter(CurrentTarget).Input := FormInput.MemoInput.Lines.Text;
  if not CurrentTarget.Compiled then AProgramCompile.Execute;
  CurrentTarget.Run;
  CurrentTarget.Messages.AddMessage(SProgramStarted);
end;

procedure TMainForm.AShowSourcePositionExecute(Sender: TObject);
var
  DebugStep: TDebugStep;
begin
  DebugStep := CurrentTarget.DebugSteps.SearchByTargetPos(FormTargetCode.MemoTarget.SelStart);
  if Assigned(DebugStep) then begin
    FormSourceCode.MemoSource.SelStart := DebugStep.SourcePosition;
    PageControlMain.TabIndex := 0;
  end;
end;

procedure TMainForm.AShowTargetPositionExecute(Sender: TObject);
var
  DebugStep: TDebugStep;
begin
  DebugStep := CurrentTarget.DebugSteps.SearchBySourcePos(FormSourceCode.MemoSource.SelStart);
  if Assigned(DebugStep) then begin
    FormTargetCode.MemoTarget.SelStart := DebugStep.TargetPosition;
    PageControlMain.TabIndex := 1;
  end;
end;

procedure TMainForm.AViewCompilersExecute(Sender: TObject);
begin
  FormCompilers.Show;
end;

procedure TMainForm.AViewInterpretterExecute(Sender: TObject);
begin
  FormCPU.Show;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  if Core.Modified then AProjectSaveAs.Execute;
end;

procedure TMainForm.AExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.AOptionsExecute(Sender: TObject);
begin
  OptionsForm.Load;
  //OptionsForm.LoadFromInterpretter(CurrentTarget);
  if OptionsForm.ShowModal = mrOK then begin
    //OptionsForm.SaveToInterpretter(CurrentTarget);
    OptionsForm.Save;
  end;
end;

procedure TMainForm.AProgramCompileExecute(Sender: TObject);
begin
  with CurrentTarget do begin
    AProgramStop.Execute;
    Optimization := coNormal;
    SourceCode := FormSourceCode.MemoSource.Text;
    ProjectFileName := Core.ProjectFileName;
    ProgramName := ExtractFileNameOnly(Core.ProjectFileName);
    CurrentTarget.Messages.AddMessage(SCompileStart);
    Compile;
    CurrentTarget.Messages.AppendMessage(SDone);
    FormTargetCode.MemoTarget.Text := TargetCode;
    UpdateInterface;
  end;
end;

procedure TMainForm.ABreakpointToggleExecute(Sender: TObject);
var
  DebugStep: TDebugStep;
  BreakPoint: TBreakPoint;
begin
  DebugStep := CurrentTarget.DebugSteps.SearchBySourcePos(FormSourceCode.MemoSource.SelStart);
  if Assigned(DebugStep) then begin
    BreakPoint := CurrentTarget.BreakPoints.SearchByTargetPos(DebugStep.TargetPosition);
    if Assigned(BreakPoint) then
      CurrentTarget.BreakPoints.Delete(CurrentTarget.BreakPoints.IndexOf(BreakPoint))
      else CurrentTarget.BreakPoints.AddItem(DebugStep.TargetPosition);
  end;
end;

procedure TMainForm.AProgramPauseExecute(Sender: TObject);
begin
  CurrentTarget.Pause;
end;

procedure TMainForm.AProgramRunToCursorExecute(Sender: TObject);
var
  DebugStep: TDebugStep;
begin
  if CurrentTarget.State = rsStopped then begin
    if not CurrentTarget.Compiled then AProgramCompile.Execute;
    DebugStep := CurrentTarget.DebugSteps.SearchBySourcePos(FormSourceCode.MemoSource.SelStart);
    CurrentTarget.BreakPoints.SetSystem(DebugStep.TargetPosition);
    AProgramRun.Execute;
  end else begin
    DebugStep := CurrentTarget.DebugSteps.SearchBySourcePos(FormSourceCode.MemoSource.SelStart);
    CurrentTarget.RunToCursor(DebugStep.TargetPosition);
  end;
end;

procedure TMainForm.AProgramShowExecutionPointExecute(Sender: TObject);
var
  DebugStep: TDebugStep;
begin
  DebugStep := CurrentTarget.DebugSteps.SearchByTargetPos(CurrentTarget.ExecutionPosition);
  FormSourceCode.MemoSource.SelStart := DebugStep.SourcePosition;
  FormTargetCode.MemoTarget.SelStart := DebugStep.TargetPosition;
  if PageControlMain.TabIndex = 0 then FormSourceCode.MemoSource.SetFocus;
  if PageControlMain.TabIndex = 1 then FormTargetCode.MemoTarget.SetFocus;
end;

procedure TMainForm.AProgramStepIntoExecute(Sender: TObject);
begin
  if CurrentTarget.State = rsStopped then begin
    if not CurrentTarget.Compiled then AProgramCompile.Execute;
    CurrentTarget.BreakPoints.SetSystem(TDebugStep(CurrentTarget.DebugSteps.First).TargetPosition);
    AProgramRun.Execute;
  end else CurrentTarget.StepInto;
end;

procedure TMainForm.AProgramStepOutExecute(Sender: TObject);
begin
  CurrentTarget.StepOut;
end;

procedure TMainForm.AProgramStepOverExecute(Sender: TObject);
begin
  if CurrentTarget.State = rsStopped then begin
    if not CurrentTarget.Compiled then AProgramCompile.Execute;
    CurrentTarget.BreakPoints.SetSystem(TDebugStep(CurrentTarget.DebugSteps.First).TargetPosition);
    AProgramRun.Execute;
  end else CurrentTarget.StepOver;
end;

procedure TMainForm.AProgramStopExecute(Sender: TObject);
begin
  CurrentTarget.Stop;
end;

procedure TMainForm.AProjectCloseExecute(Sender: TObject);
begin
  if Core.Modified then AProjectSaveAs.Execute;
  AProgramStop.Execute;
  Core.ProjectFileName := '';
  FormSourceCode.MemoSource.Clear;
  Core.Modified := False;
  UpdateInterface;
end;

procedure TMainForm.AProjectNewExecute(Sender: TObject);
begin
  AProjectClose.Execute;
  Core.ProjectFileName := 'Examples' + DirectorySeparator + SNewProject + '.b';
  FormSourceCode.MemoSource.Clear;
  Core.Modified := False;
  CurrentTarget.Compiled := False;
  UpdateInterface;
end;

procedure TMainForm.AProjectOpenRecentExecute(Sender: TObject);
begin
  ProjectOpen(Core.LastOpenedList.Items[TMenuItem(Sender).MenuIndex]);
end;

procedure TMainForm.AProjectOpenExecute(Sender: TObject);
begin
  AProjectClose.Execute;
  OpenDialog1.FileName := LastFileName;
  OpenDialog1.InitialDir := ExtractFileDir(LastFileName);
  if OpenDialog1.Execute then begin
    LastFileName := OpenDialog1.FileName;
    ProjectOpen(OpenDialog1.FileName);
  end;
end;

procedure TMainForm.AProjectSaveAsExecute(Sender: TObject);
begin
  SaveDialog1.FileName := Core.ProjectFileName;
  if SaveDialog1.Execute then begin
    FormSourceCode.MemoSource.Lines.SaveToFile(UTF8Decode(SaveDialog1.FileName));
    Core.ProjectFileName := SaveDialog1.FileName;
    Core.LastOpenedList.AddItem(SaveDialog1.FileName);
    Core.Modified := False;
    UpdateInterface;
  end;
end;

procedure TMainForm.AProjectSaveExecute(Sender: TObject);
begin
  if Core.Modified then AProjectSaveAs.Execute;
end;

end.

