unit UFormOptions;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  Spin, UTargetInterpretter;

type

  { TOptionsForm }

  TOptionsForm = class(TForm)
    ButtonOk: TButton;
    ButtonCancel: TButton;
    CheckBox1: TCheckBox;
    CheckBoxDPIAuto: TCheckBox;
    ComboBoxLanguage: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    LabelDPI: TLabel;
    LabelX: TLabel;
    SpinEditDPIX: TSpinEdit;
    SpinEditDPIY: TSpinEdit;
    SpinEditCellSize: TSpinEdit;
    SpinEditMemorySize: TSpinEdit;
    procedure ButtonOkClick(Sender: TObject);
    procedure CheckBoxDPIAutoChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { private declarations }
  public
    procedure UpdateInterface;
    procedure Load;
    procedure Save;
    procedure LoadFromInterpretter(Interpretter: TTargetInterpretter);
    procedure SaveToInterpretter(Interpretter: TTargetInterpretter);
  end;

var
  OptionsForm: TOptionsForm;

implementation

{$R *.lfm}

uses
  UCore, ULanguages;

{ TOptionsForm }

procedure TOptionsForm.FormShow(Sender: TObject);
begin
  Core.CoolTranslator1.LanguageListToStrings(ComboBoxLanguage.Items);
  ComboBoxLanguage.ItemIndex := ComboBoxLanguage.Items.IndexOfObject(Core.CoolTranslator1.Language);
  if ComboBoxLanguage.ItemIndex = -1 then ComboBoxLanguage.ItemIndex := 0;
  CheckBox1.Checked := Core.OpenProjectOnStart;
end;

procedure TOptionsForm.UpdateInterface;
begin
  SpinEditDPIX.Enabled := not CheckBoxDPIAuto.Checked;
  SpinEditDPIY.Enabled := not CheckBoxDPIAuto.Checked;
  LabelDPI.Enabled := not CheckBoxDPIAuto.Checked;
  LabelX.Enabled := not CheckBoxDPIAuto.Checked;
  if CheckBoxDPIAuto.Checked then begin
    SpinEditDPIX.Value := ScreenInfo.PixelsPerInchX;
    SpinEditDPIY.Value := ScreenInfo.PixelsPerInchY;
  end;
end;

procedure TOptionsForm.Load;
begin
  SpinEditDPIX.Value := Core.ScaleDPI.DPI.X;
  SpinEditDPIY.Value := Core.ScaleDPI.DPI.Y;
  CheckBoxDPIAuto.Checked := Core.ScaleDPI.AutoDetect;
  UpdateInterface;
end;

procedure TOptionsForm.Save;
begin
  Core.ScaleDPI.DPI.X := SpinEditDPIX.Value;
  Core.ScaleDPI.DPI.Y := SpinEditDPIY.Value;
  Core.ScaleDPI.AutoDetect := CheckBoxDPIAuto.Checked;
end;

procedure TOptionsForm.ButtonOkClick(Sender: TObject);
begin
  if ComboBoxLanguage.ItemIndex <> -1 then
    Core.CoolTranslator1.Language := TLanguage(ComboBoxLanguage.Items.Objects[ComboBoxLanguage.ItemIndex]);
  Core.OpenProjectOnStart := CheckBox1.Checked;
end;

procedure TOptionsForm.CheckBoxDPIAutoChange(Sender: TObject);
begin
  UpdateInterface;
end;

procedure TOptionsForm.LoadFromInterpretter(Interpretter: TTargetInterpretter
  );
begin
  SpinEditCellSize.Value := Interpretter.CellSize;
  SpinEditMemorySize.Value := Interpretter.MemorySize;
end;

procedure TOptionsForm.SaveToInterpretter(Interpretter: TTargetInterpretter);
begin
  Interpretter.CellSize := SpinEditCellSize.Value;
  Interpretter.MemorySize := SpinEditMemorySize.Value;
end;

end.

